@@ -1,11 +1,21 @@
 import pytest
 
 from hatchling.metadata.core import BuildMetadata, CoreMetadata, HatchMetadata, ProjectMetadata
+from hatchling.metadata.spec import (
+    LATEST_METADATA_VERSION,
+    get_core_metadata_constructors,
+    project_metadata_from_core_metadata,
+)
 from hatchling.plugin.manager import PluginManager
 from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT
 from hatchling.version.source.regex import RegexSource
 
 
+@pytest.fixture(scope='module')
+def latest_spec():
+    return get_core_metadata_constructors()[LATEST_METADATA_VERSION]
+
+
 class TestConfig:
     def test_default(self, isolation):
         metadata = ProjectMetadata(str(isolation), None)
@@ -32,13 +42,8 @@ class TestInterface:
     def test_types(self, isolation):
         metadata = ProjectMetadata(str(isolation), None, {'project': {}})
 
-        assert metadata.core is metadata._core
         assert isinstance(metadata.core, CoreMetadata)
-
-        assert metadata.hatch is metadata._hatch
         assert isinstance(metadata.hatch, HatchMetadata)
-
-        assert metadata.build is metadata._build
         assert isinstance(metadata.build, BuildMetadata)
 
     def test_missing_core_metadata(self, isolation):
@@ -89,7 +94,6 @@ class TestDynamic:
         dynamic = ['version']
         metadata = ProjectMetadata(str(isolation), None, {'project': {'dynamic': dynamic}})
 
-        assert metadata.core.dynamic is dynamic
         assert metadata.core.dynamic == ['version']
 
     def test_cache_not_array(self, isolation):
@@ -383,7 +387,7 @@ class TestReadme:
             _ = metadata.core.readme
 
     @pytest.mark.parametrize(
-        'extension, content_type', [('.md', 'text/markdown'), ('.rst', 'text/x-rst'), ('.txt', 'text/plain')]
+        ('extension', 'content_type'), [('.md', 'text/markdown'), ('.rst', 'text/x-rst'), ('.txt', 'text/plain')]
     )
     def test_string_correct(self, extension, content_type, temp_dir):
         metadata = ProjectMetadata(str(temp_dir), None, {'project': {'readme': f'foo/bar{extension}'}})
@@ -1080,14 +1084,15 @@ class TestEntryPoints:
         with pytest.raises(TypeError, match='Field `project.entry-points` must be a table'):
             _ = metadata.core.entry_points
 
-    @pytest.mark.parametrize('field', ['scripts', 'gui-scripts'])
-    def test_forbidden_fields(self, isolation, field):
+    @pytest.mark.parametrize(('field', 'expected'), [('console_scripts', 'scripts'), ('gui-scripts', 'gui-scripts')])
+    def test_forbidden_fields(self, isolation, field, expected):
         metadata = ProjectMetadata(str(isolation), None, {'project': {'entry-points': {field: 'foo'}}})
 
         with pytest.raises(
             ValueError,
             match=(
-                f'Field `{field}` must be defined as `project.{field}` instead of in the `project.entry-points` table'
+                f'Field `{field}` must be defined as `project.{expected}` instead of '
+                f'in the `project.entry-points` table'
             ),
         ):
             _ = metadata.core.entry_points
@@ -1301,6 +1306,32 @@ class TestOptionalDependencies:
         ):
             _ = metadata.core.optional_dependencies
 
+    def test_recursive_circular(self, isolation):
+        metadata = ProjectMetadata(
+            str(isolation),
+            None,
+            {'project': {'name': 'my-app', 'optional-dependencies': {'foo': ['my-app[bar]'], 'bar': ['my-app[foo]']}}},
+        )
+
+        with pytest.raises(
+            ValueError,
+            match='Field `project.optional-dependencies` defines a circular dependency group: foo',
+        ):
+            _ = metadata.core.optional_dependencies
+
+    def test_recursive_unknown(self, isolation):
+        metadata = ProjectMetadata(
+            str(isolation),
+            None,
+            {'project': {'name': 'my-app', 'optional-dependencies': {'foo': ['my-app[bar]']}}},
+        )
+
+        with pytest.raises(
+            ValueError,
+            match='Unknown recursive dependency group in field `project.optional-dependencies`: bar',
+        ):
+            _ = metadata.core.optional_dependencies
+
     def test_allow_ambiguity(self, isolation):
         metadata = ProjectMetadata(
             str(isolation),
@@ -1334,7 +1365,7 @@ class TestOptionalDependencies:
             str(isolation),
             None,
             {
-                'project': {'optional-dependencies': {'foo': ['proj @ {root:uri}']}},
+                'project': {'name': 'my-app', 'optional-dependencies': {'foo': ['proj @ {root:uri}']}},
                 'tool': {'hatch': {'metadata': {'allow-direct-references': True}}},
             },
         )
@@ -1347,7 +1378,10 @@ class TestOptionalDependencies:
             str(isolation),
             None,
             {
-                'project': {'optional-dependencies': {'foo': ['proj @ git+https://github.com/org/proj.git@v1']}},
+                'project': {
+                    'name': 'my-app',
+                    'optional-dependencies': {'foo': ['proj @ git+https://github.com/org/proj.git@v1']},
+                },
                 'tool': {'hatch': {'metadata': {'allow-direct-references': True}}},
             },
         )
@@ -1360,14 +1394,19 @@ class TestOptionalDependencies:
             None,
             {
                 'project': {
+                    'name': 'my-app',
                     'optional-dependencies': {
                         'foo': [
                             'python___dateutil;platform_python_implementation=="CPython"',
                             'bAr.Baz[TLS, Zu.Bat, EdDSA, Zu_Bat]   >=1.2RC5 , <9000B1',
                             'Foo;python_version<"3.8"',
                             'fOO;     python_version<    "3.8"',
+                            'MY-APP[zZz]',
                         ],
                         'bar': ['foo', 'bar', 'Baz'],
+                        'baz': ['my___app[XYZ]'],
+                        'xyz': ['my...app[Bar]'],
+                        'zzz': ['aaa'],
                     },
                 },
             },
@@ -1378,11 +1417,15 @@ class TestOptionalDependencies:
             == metadata.core.optional_dependencies
             == {
                 'bar': ['bar', 'baz', 'foo'],
+                'baz': ['bar', 'baz', 'foo'],
                 'foo': [
+                    'aaa',
                     'bar-baz[eddsa,tls,zu-bat]<9000b1,>=1.2rc5',
                     "foo; python_version < '3.8'",
                     "python-dateutil; platform_python_implementation == 'CPython'",
                 ],
+                'xyz': ['bar', 'baz', 'foo'],
+                'zzz': ['aaa'],
             }
         )
 
@@ -1484,7 +1527,7 @@ class TestHook:
 
 
 class TestHatchPersonalProjectConfigFile:
-    def test_correct(self, isolation, temp_dir, helpers):
+    def test_correct(self, temp_dir, helpers):
         metadata = ProjectMetadata(
             str(temp_dir),
             PluginManager(),
@@ -1512,7 +1555,7 @@ class TestHatchPersonalProjectConfigFile:
         assert metadata.version == '0.0.1'
         assert metadata.hatch.build_config['reproducible'] is False
 
-    def test_precedence(self, isolation, temp_dir, helpers):
+    def test_precedence(self, temp_dir, helpers):
         metadata = ProjectMetadata(
             str(temp_dir),
             PluginManager(),
@@ -1543,3 +1586,226 @@ class TestHatchPersonalProjectConfigFile:
 
         assert metadata.version == '0.0.2'
         assert metadata.hatch.build_config['reproducible'] is False
+
+
+class TestMetadataConversion:
+    def test_required_only(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1'}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_dynamic(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'dynamic': ['authors', 'classifiers']}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_description(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'description': 'foo bar'}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_urls(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'urls': {'foo': 'bar', 'bar': 'baz'}}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_authors(self, isolation, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'authors': [{'name': 'foobar'}, {'email': 'bar@domain', 'name': 'foo'}, {'email': 'baz@domain'}],
+        }
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_maintainers(self, isolation, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'maintainers': [{'name': 'foobar'}, {'email': 'bar@domain', 'name': 'foo'}, {'email': 'baz@domain'}],
+        }
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_keywords(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'keywords': ['bar', 'foo']}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_classifiers(self, isolation, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'classifiers': ['Programming Language :: Python :: 3.9', 'Programming Language :: Python :: 3.11'],
+        }
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_license_files(self, temp_dir, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'license-files': {'paths': ['LICENSES/Apache-2.0.txt', 'LICENSES/MIT.txt']},
+        }
+        metadata = ProjectMetadata(str(temp_dir), None, {'project': raw_metadata})
+
+        licenses_path = temp_dir / 'LICENSES'
+        licenses_path.mkdir()
+        licenses_path.joinpath('Apache-2.0.txt').touch()
+        licenses_path.joinpath('MIT.txt').touch()
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_license_expression(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'license': 'MIT'}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_license_legacy(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'license': {'text': 'foo'}}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_readme(self, isolation, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'readme': {'content-type': 'text/markdown', 'text': 'test content\n'},
+        }
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_requires_python(self, isolation, latest_spec):
+        raw_metadata = {'name': 'My.App', 'version': '0.0.1', 'requires-python': '<2,>=1'}
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+    def test_dependencies(self, isolation, latest_spec):
+        raw_metadata = {
+            'name': 'My.App',
+            'version': '0.0.1',
+            'dependencies': ['bar==5', 'foo==1'],
+            'optional-dependencies': {
+                'feature1': ['bar==5; python_version < "3"', 'foo==1'],
+                'feature2': ['bar==5', 'foo==1; python_version < "3"'],
+            },
+        }
+        metadata = ProjectMetadata(str(isolation), None, {'project': raw_metadata})
+
+        core_metadata = latest_spec(metadata)
+        assert project_metadata_from_core_metadata(core_metadata) == raw_metadata
+
+
+class TestSourceDistributionMetadata:
+    def test_basic_persistence(self, temp_dir, helpers):
+        metadata = ProjectMetadata(
+            str(temp_dir),
+            None,
+            {
+                'project': {
+                    'name': 'My.App',
+                    'dynamic': ['version', 'keywords'],
+                    'dependencies': ['foo==1'],
+                    'scripts': {'foo': 'bar'},
+                },
+            },
+        )
+
+        pkg_info = temp_dir / 'PKG-INFO'
+        pkg_info.write_text(
+            helpers.dedent(
+                f"""
+                Metadata-Version: {LATEST_METADATA_VERSION}
+                Name: My.App
+                Version: 0.0.1
+                Keywords: foo,bar
+                Requires-Dist: bar==5
+                """
+            )
+        )
+        with temp_dir.as_cwd():
+            assert metadata.core.config == {
+                'name': 'My.App',
+                'version': '0.0.1',
+                'dependencies': ['foo==1'],
+                'keywords': ['foo', 'bar'],
+                'scripts': {'foo': 'bar'},
+                'dynamic': [],
+            }
+
+    def test_metadata_hooks(self, temp_dir, helpers):
+        metadata = ProjectMetadata(
+            str(temp_dir),
+            PluginManager(),
+            {
+                'project': {
+                    'name': 'My.App',
+                    'dynamic': ['version', 'keywords', 'description', 'scripts'],
+                    'dependencies': ['foo==1'],
+                },
+                'tool': {'hatch': {'metadata': {'hooks': {'custom': {}}}}},
+            },
+        )
+
+        build_script = temp_dir / DEFAULT_BUILD_SCRIPT
+        build_script.write_text(
+            helpers.dedent(
+                """
+                from hatchling.metadata.plugin.interface import MetadataHookInterface
+
+                class CustomHook(MetadataHookInterface):
+                    def update(self, metadata):
+                        metadata['description'] = metadata['name'] + ' bar'
+                        metadata['scripts'] = {'foo': 'bar'}
+                """
+            )
+        )
+
+        pkg_info = temp_dir / 'PKG-INFO'
+        pkg_info.write_text(
+            helpers.dedent(
+                f"""
+                Metadata-Version: {LATEST_METADATA_VERSION}
+                Name: My.App
+                Version: 0.0.1
+                Summary: My.App bar
+                Keywords: foo,bar
+                Requires-Dist: bar==5
+                """
+            )
+        )
+        with temp_dir.as_cwd():
+            assert metadata.core.config == {
+                'name': 'My.App',
+                'version': '0.0.1',
+                'description': 'My.App bar',
+                'dependencies': ['foo==1'],
+                'keywords': ['foo', 'bar'],
+                'scripts': {'foo': 'bar'},
+                'dynamic': ['scripts'],
+            }
