@@ -2,16 +2,13 @@ from hatch.template import File
 from hatch.utils.fs import Path
 from hatchling.metadata.spec import DEFAULT_METADATA_VERSION
 
-from ..new.default import get_files as get_template_files
+from ..new.feature_no_src_layout import get_files as get_template_files
 
 
 def get_files(**kwargs):
     relative_root = kwargs.get('relative_root', '')
 
-    files = []
-    for f in get_template_files(**kwargs):
-        files.append(File(Path(relative_root, f.path), f.contents))
-
+    files = [File(Path(relative_root, f.path), f.contents) for f in get_template_files(**kwargs)]
     files.append(
         File(
             Path(relative_root, 'PKG-INFO'),
