@@ -3,18 +3,15 @@ from hatch.utils.fs import Path
 from hatchling.metadata.spec import DEFAULT_METADATA_VERSION
 from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT
 
-from ..new.default import get_files as get_template_files
+from ..new.feature_no_src_layout import get_files as get_template_files
 
 
 def get_files(**kwargs):
     relative_root = kwargs.get('relative_root', '')
 
-    files = []
-    for f in get_template_files(**kwargs):
-        files.append(File(Path(relative_root, f.path), f.contents))
-
-    files.append(File(Path(relative_root, kwargs['package_name'], 'lib.so'), ''))
-    files.append(
+    files = [File(Path(relative_root, f.path), f.contents) for f in get_template_files(**kwargs)]
+    files.extend((
+        File(Path(relative_root, kwargs['package_name'], 'lib.so'), ''),
         File(
             Path(relative_root, '.gitignore'),
             """\
@@ -22,9 +19,7 @@ def get_files(**kwargs):
 *.so
 *.h
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(relative_root, DEFAULT_BUILD_SCRIPT),
             """\
@@ -38,9 +33,7 @@ class CustomHook(BuildHookInterface):
         pathlib.Path('my_app', 'lib.h').touch()
         build_data['dependencies'].append('binary')
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(relative_root, 'PKG-INFO'),
             f"""\
@@ -50,7 +43,7 @@ Version: 0.0.1
 License-File: LICENSE.txt
 Requires-Dist: binary
 """,
-        )
-    )
+        ),
+    ))
 
     return files
