@@ -8,12 +8,9 @@ from ..new.feature_no_src_layout import get_files as get_template_files
 def get_files(**kwargs):
     relative_root = kwargs.get('relative_root', '')
 
-    files = []
-    for f in get_template_files(**kwargs):
-        files.append(File(Path(relative_root, f.path), f.contents))
-
-    files.append(File(Path(relative_root, kwargs['package_name'], 'lib.so'), ''))
-    files.append(
+    files = [File(Path(relative_root, f.path), f.contents) for f in get_template_files(**kwargs)]
+    files.extend((
+        File(Path(relative_root, kwargs['package_name'], 'lib.so'), ''),
         File(
             Path(relative_root, '.hgignore'),
             """\
@@ -27,9 +24,7 @@ syntax: glob
 *.so
 *.h
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(relative_root, 'PKG-INFO'),
             f"""\
@@ -38,7 +33,7 @@ Name: {kwargs['project_name']}
 Version: 0.0.1
 License-File: LICENSE.txt
 """,
-        )
-    )
+        ),
+    ))
 
     return files
