@@ -20,9 +20,9 @@ def get_files(**kwargs):
 
         files.append(f)
 
-    files.append(File(Path(kwargs['package_name'], 'lib.so'), ''))
-    files.append(File(Path(kwargs['package_name'], 'lib.h'), ''))
-    files.append(
+    files.extend((
+        File(Path(kwargs['package_name'], 'lib.so'), ''),
+        File(Path(kwargs['package_name'], 'lib.h'), ''),
         File(
             Path(metadata_directory, 'WHEEL'),
             f"""\
@@ -31,9 +31,7 @@ Generator: hatchling {__version__}
 Root-Is-Purelib: false
 Tag: {kwargs.get('tag', '')}
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(metadata_directory, 'METADATA'),
             f"""\
@@ -43,8 +41,8 @@ Version: 0.0.1
 License-File: LICENSE.txt
 Requires-Python: >3
 """,
-        )
-    )
+        ),
+    ))
 
     record_file = File(Path(metadata_directory, 'RECORD'), '')
     update_record_file_contents(record_file, files)
