@@ -13,14 +13,14 @@ def get_files(**kwargs):
     files = []
     for f in get_template_files(**kwargs):
         if str(f.path) == 'LICENSE.txt':
-            files.append(File(Path(metadata_directory, 'license_files', f.path), f.contents))
+            files.append(File(Path(metadata_directory, 'licenses', f.path), f.contents))
 
-        if f.path.parts[0] != kwargs['package_name']:
+        if f.path.parts[0] != 'src':
             continue
 
-        files.append(f)
+        files.append(File(Path(*f.path.parts[1:]), f.contents))
 
-    files.append(
+    files.extend((
         File(
             Path(metadata_directory, 'WHEEL'),
             f"""\
@@ -30,18 +30,17 @@ Root-Is-Purelib: true
 Tag: py2-none-any
 Tag: py3-none-any
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(metadata_directory, 'METADATA'),
             f"""\
 Metadata-Version: {DEFAULT_METADATA_VERSION}
-Name: {kwargs['project_name_normalized']}
+Name: {kwargs['project_name']}
 Version: 0.0.1
+License-File: LICENSE.txt
 """,
-        )
-    )
+        ),
+    ))
 
     record_file = File(Path(metadata_directory, 'RECORD'), '')
     update_record_file_contents(record_file, files)
