@@ -3,7 +3,7 @@ from hatch.utils.fs import Path
 from hatchling.__about__ import __version__
 from hatchling.metadata.spec import DEFAULT_METADATA_VERSION
 
-from ..new.default import get_files as get_template_files
+from ..new.feature_no_src_layout import get_files as get_template_files
 from .utils import update_record_file_contents
 
 
@@ -20,8 +20,8 @@ def get_files(**kwargs):
 
         files.append(f)
 
-    files.append(File(Path(kwargs['package_name'], 'lib.so'), 'data'))
-    files.append(
+    files.extend((
+        File(Path(kwargs['package_name'], 'lib.so'), 'data'),
         File(
             Path(metadata_directory, 'WHEEL'),
             f"""\
@@ -30,9 +30,7 @@ Generator: hatchling {__version__}
 Root-Is-Purelib: false
 Tag: {kwargs.get('tag', '')}
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(metadata_directory, 'METADATA'),
             f"""\
@@ -42,8 +40,8 @@ Version: 0.0.1
 License-File: LICENSE.txt
 Requires-Python: >3
 """,
-        )
-    )
+        ),
+    ))
 
     record_file = File(Path(metadata_directory, 'RECORD'), '')
     update_record_file_contents(record_file, files)
