@@ -3,7 +3,7 @@ from hatch.utils.fs import Path
 from hatchling.__about__ import __version__
 from hatchling.metadata.spec import DEFAULT_METADATA_VERSION
 
-from ..new.default import get_files as get_template_files
+from ..new.feature_no_src_layout import get_files as get_template_files
 from .utils import update_record_file_contents
 
 
@@ -15,14 +15,15 @@ def get_files(**kwargs):
         if str(f.path) == 'LICENSE.txt':
             files.append(File(Path(metadata_directory, 'licenses', f.path), f.contents))
 
-        if f.path.parts[0] not in (kwargs['package_name'], 'tests'):
+        if f.path.parts[0] not in {kwargs['package_name'], 'tests'}:
             continue
-        elif f.path == Path('tests', '__init__.py'):
+
+        if f.path == Path('tests', '__init__.py'):
             f.path = Path('tests', 'foo.py')
 
         files.append(f)
 
-    files.append(
+    files.extend((
         File(
             Path(metadata_directory, 'WHEEL'),
             f"""\
@@ -32,9 +33,7 @@ Root-Is-Purelib: true
 Tag: py2-none-any
 Tag: py3-none-any
 """,
-        )
-    )
-    files.append(
+        ),
         File(
             Path(metadata_directory, 'METADATA'),
             f"""\
@@ -43,8 +42,8 @@ Name: {kwargs['project_name']}
 Version: 0.0.1
 License-File: LICENSE.txt
 """,
-        )
-    )
+        ),
+    ))
 
     record_file = File(Path(metadata_directory, 'RECORD'), '')
     update_record_file_contents(record_file, files)
